Object.defineProperty(exports, "__esModule", { value: true });
var os_1 = require("os");
var console_1 = require("./console");
var exception_1 = require("../exception");
exports.assertSnapshot = function (snapshot) {
    if (snapshot == null) {
        throw new exception_1.OutputException('Cannot output a null application snapshot');
    }
    switch (snapshot.exceptions.length) {
        case 0: break;
        case 1: throw snapshot.exceptions[0];
        default:
            throw new exception_1.AggregateException(snapshot.exceptions);
    }
    if (snapshot.renderedDocument == null || snapshot.renderedDocument.length === 0) {
        throw new exception_1.OutputException('Received an application snapshot with an empty document!');
    }
    var logs = snapshot.console || [];
    var errors = logs.filter(function (c) { return c.type === console_1.ConsoleType.Error || c.type === console_1.ConsoleType.Assertion; });
    if (errors.length > 0) {
        var messages = errors.map(function (message) { return message.args.map(function (a) { return a.toString(); }).join(' '); }).join(os_1.EOL);
        throw new exception_1.OutputException("Application printed error messages to the console, which is considered a failure: " + messages);
    }
};
//# sourceMappingURL=assert.js.map